/*
 *        id3info.c
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <sp/spBaseLib.h>
#include <sp/spWave.h>

#include <sp/spID3.h>
#include <sp/spMain.h>

#define DEBUG

static spBool display_song_info;
static spBool help_flag;
static long file_offset = 0L;
static int debug_level = -1;

static char o_id3_file[SP_MAX_PATHNAME] = "";

static spOptions options;
static spOption option[] = {
    {"-song", NULL, "display song information", NULL,
         SP_TYPE_BOOLEAN, &display_song_info, SP_FALSE_STRING},
    {"-offset", NULL, "file offset before ID3 tag", NULL,
         SP_TYPE_LONG, &file_offset, "0"},
    {"-O", NULL, "output file (binary ID3 content)", NULL,
         SP_TYPE_STRING_A, o_id3_file, NULL},
    {"-debug", NULL, "debug level", NULL,
         SP_TYPE_INT, &debug_level, NULL},
    {"-help", "-h", "display this message", NULL,
         SP_TYPE_BOOLEAN, &help_flag, SP_FALSE_STRING},
};

static const char *filelabel[] = {
    "<filename...>",
};

static void showID3FrameString(long depth, char encoding, unsigned char *string)
{
    long k;
    char buf[SP_MAX_LINE];
    
    spDebug(10, "showID3FrameString", "encoding = %d\n", encoding);
    
    if (encoding == 1 || encoding == 2) {
        spUnicode16ToUTF8((unsigned short *)string, (unsigned char *)buf, sizeof(buf));
    } else {
        spStrCopy(buf, sizeof(buf), (char *)string);
    }
            
    if (encoding != 0) {
        spConvertKanjiToLocaleCode((unsigned char *)buf, sizeof(buf), SP_KANJI_CODE_UTF8);
    }
    for (k = 1; k <= depth; k++) printf("  ");
    printf("String (encoding = %d): '%s'\n", encoding, buf);
    
    return;
}

static void showID3FrameInfo(spID3Frame *frame, long depth)
{
    long k;
    char type_string[5];
    spID3Frame *child_frame;
    spID3FrameSize frame_size;

    frame_size = spGetID3FrameSize(frame, type_string);
    spDebug(10, "showID3FrameInfo", "depth = %ld, frame_size = %ld\n", depth, (long)frame_size);

    if (frame_size > 0) {
        for (k = 1; k < depth; k++) printf("  ");
        printf("'%s' (%ld bytes)\n", type_string, (long)frame_size);

        if (strneq(type_string, "ID3", 3)) {
            spID3Header *header = (spID3Header *)frame;
            printf("  V2.%d.%d\n", header->version, header->revision);
        } else if (strneq(type_string, "COMM", 4) || strneq(type_string, "COM ", 4)
                   || strneq(type_string, "USLT", 4)|| strneq(type_string, "ULT ", 4)) {
            spID3LangTextFrame *ltframe = (spID3LangTextFrame *)frame;
            
            showID3FrameString(depth, ltframe->encoding, ltframe->string);
            if (ltframe->actual_string != NULL) {
                showID3FrameString(depth, ltframe->encoding, ltframe->actual_string);
            }
        } else if (strneq(type_string, "T", 1) || strneq(type_string, "COMM", 4)
            || strneq(type_string, "COM ", 4)) {
            spID3TextFrame *tframe = (spID3TextFrame *)frame;

            showID3FrameString(depth, tframe->encoding, tframe->string);
        }
    }
    
    child_frame = frame->child;
    while (child_frame != NULL) {
        showID3FrameInfo(child_frame, depth + 1);
        child_frame = child_frame->next;
    }

    return;
}

int spMain(int argc, char *argv[])
{
    const char *filename;
    FILE *fp;
    long file_count;
    spID3FrameSize nread;
    spID3FrameSize total_read;
    spID3Header id3_header;
    spID3Header o_id3_header;
    spSongInfoV2 song_info;
    
    spSetHelpMessage(&help_flag, "Get ID3 file information (sample program)");
    options = spGetOptions(argc, argv, option, filelabel);
    spGetOptionsValue(argc, argv, options);

    spSetDebugLevel(debug_level);
    
    filename = spGetFile(options);
    if (filename == NULL) {
        spError(1, "Not enough files.\n");
    }

    file_count = 0;
    do {
        /* open file */
        if (NULL == (fp = spOpenFile(filename, "rb"))) {
            return 1;
        }
        
        printf("%s:\n", filename);
        total_read = 0;

        if (file_offset > 0) {
            spSeekFile(fp, file_offset, SEEK_CUR);
        }
        
        /* read information */
        if ((nread = spReadID3Header(&id3_header, fp)) <= 0) {
            printf("  ");
            printf("This file is not ID3 file.\n");
        } else {
            total_read += nread;

            showID3FrameInfo((spID3Frame *)&id3_header, 0);

            if (display_song_info == SP_TRUE) {
                if (spParseID3SongInfo(&id3_header, &song_info)) {
                    printf("\n");
                    spShowSongInfoV2(&song_info, NULL, stdout);
                    printf("\n");
                }
            }
            
            if (file_count == 0) {
                spCopyID3Header(&o_id3_header, &id3_header);
            }
            
            spFreeID3Header(&id3_header);
        
            ++file_count;
        }
    
        /* close file */
        spCloseFile(fp);
    } while ((filename = spGetFile(options)) != NULL);

    if (!spStrNone(o_id3_file)) {
        /* open file */
        if (NULL == (fp = spOpenFile(o_id3_file, "wb"))) {
        } else {
            spID3FrameSize nwrite;
            
            printf("Output ID3 File: %s:\n", o_id3_file);

#if 1
            showID3FrameInfo((spID3Frame *)&o_id3_header, 0);
#endif

            nwrite = spWriteID3Header(&o_id3_header, fp);
            spDebug(-10, "main", "nwrite = %ld\n", (long)nwrite);

            spCloseFile(fp);
        }
    }
    
    return 0;
}
